﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=12.8
@EndOfDesignText@
Sub 	Class_Globals
	Type ThemeColor (LightColor As Int, DarkColor As Int)
		
	Public IsDark As Boolean
	Public Const TMODE_FOLLOW_SYSTEM As Int = -1
	Public Const TMODE_LIGHT_MODE As Int = 1
	Public Const TMODE_NIGHT_MODE As Int = 2
	
	Private Const FileName As String = "AppTheme.bin"

	Private mTMode As Int = -1
	Private ThemeMap, DataMap, SettingsMap As Map
	Private mDir As String
	Private xui As XUI
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize (vDir As String)
	File.Delete(vDir, FileName)
	mDir = vDir
	
	ThemeMap.Initialize
	DataMap.Initialize
	SettingsMap.Initialize
	
	mTMode = TMODE_FOLLOW_SYSTEM
	
	ReadThemeMap	
	DetectDarkMode
End Sub

Public Sub CreateThemeColor (LightColor As Int, DarkColor As Int) As ThemeColor
	Dim TC As ThemeColor
	TC.Initialize
	TC.LightColor = LightColor
	TC.DarkColor = DarkColor
	Return TC	
End Sub

'Forces the theme type. Use the AppTheme.TMODE helpers for the TMode variable.
Public Sub setTheme (TMode As Int)
	mTMode = TMode
	SettingsMap.Put("Theme", mTMode)
End Sub

Public Sub getTheme As Int
	mTMode = SettingsMap.GetDefault("Theme", TMODE_FOLLOW_SYSTEM)
	Return mTMode
End Sub

Public Sub getATMap As Map
	Return ThemeMap
End Sub

Private Sub DetectDarkMode
	Dim RB As Boolean
	#If B4A
	Dim ctxt As JavaObject
	ctxt.InitializeContext
	RB = Bit.And(ctxt.RunMethodJO("getResources", Null).RunMethodJO("getConfiguration", Null).GetField("uiMode"), 0x30) = 0x20
	#End If
	#If B4i
	Dim App As Application
    If App.OSVersion < 13 Then IsDark = False
    Dim No As NativeObject = App.KeyController
    RB = No.GetField("traitCollection").GetField("userInterfaceStyle").AsNumber = 2
	#End If
	
	Select mTMode
		Case TMODE_FOLLOW_SYSTEM
			IsDark = RB
		Case TMODE_LIGHT_MODE
			IsDark = False
		Case TMODE_NIGHT_MODE
			IsDark = True
	End Select
End Sub

'Puts a ThemeColor into the provided group.
'To use the Default group, just pass a blank string ("") as the GroupName or use SimplePut (+SimpleGet).
Public Sub PutColor (Key As String, TC As ThemeColor)
	ReadThemeMap
	DataMap.Put(Key, TC)
	WriteThemeMap
End Sub

Public Sub GetColor (Key As String) As Int
	ReadThemeMap
	
	Dim TC As ThemeColor = DataMap.GetDefault(Key, CreateThemeColor(xui.Color_White, xui.Color_Black))	
	Return IIf(IsDark, TC.DarkColor, TC.LightColor)
End Sub

Private Sub WriteThemeMap
	Dim Ser As B4XSerializator
	ThemeMap.Put("ThemeData", DataMap)
	ThemeMap.Put("Settings", SettingsMap)
	File.WriteBytes(mDir, FileName, Ser.ConvertObjectToBytes(ThemeMap))
End Sub

Private Sub ReadThemeMap
	If File.Exists(mDir, FileName) Then
		Dim Ser As B4XSerializator
		ThemeMap = Ser.ConvertBytesToObject(File.ReadBytes(mDir, FileName))
		DataMap = ThemeMap.GetDefault("ThemeData", CreateMap())
		SettingsMap = ThemeMap.GetDefault("Settings", CreateMap())
		
	End If
End Sub